% Simulation on mutual coupling effect over a practical setting
clear; clc; close all;

SIZE = 16;
aratio = [3, 1, 1];

% ==================== SNR ====================
SNRdB = 0;

% ==================== Snapshots ====================
SNAPSHOTS = 500;

% ==================== Source distribution ====================
DD = 20; theta_bar_max = 0.2;
theta_bar = linspace(-theta_bar_max, theta_bar_max, DD).';
amp = ones(DD, 1);


% In this example, we consider an array with 34 sensors
for type = 1 : 6
    switch(type)
        case 1
            % ULA
            S = (0 : 13).';
            title_str = 'ULA';
            filename = 'ULA';
            
            if (DD >= length(S))
                continue;
            end
        case 2
            % MRA
            S = cumsum([0, 1, 5, 8, 8, 8, 8, 8, 8, 2, 2, 3, 2, 2].');
            [n1, n2] = ndgrid(S);
            n1_n2_mat = n1 - n2;
            n1_n2_vec = n1_n2_mat(:);
            D = unique(n1_n2_vec);
            LEN_D = length(D);
            U = D;
            
            title_str = 'MRA';
            filename = 'MRA';
        case 3
            % Nested arrays
            N1 = 7; N2 = 7;
            LEN_D = 2*((N1+1)*N2-1)+1;
            S = super_nested(N1, N2, 1);
            [n1, n2] = ndgrid(S);
            n1_n2_mat = n1 - n2;
            n1_n2_vec = n1_n2_mat(:);
            D = unique(n1_n2_vec);
            U = D;
            
            title_str = 'Nested arrays';
            filename = 'Nested';
            
            
        case 4
            % Coprime arrays
            M = 4; N = 7;
            LEN_D = 3*M*N+M-N;
            S = sort([(0:N-1)*M, (1:2*M-1)*N].', 'ascend');
            [n1, n2] = ndgrid(S);
            n1_n2_mat = n1 - n2;
            n1_n2_vec = n1_n2_mat(:);
            D = unique(n1_n2_vec);
            U = (-(M*N+M-1) : (M*N+M-1)).';
            
            title_str = 'Coprime arrays';
            filename = 'Coprime';            
        case 5
            % 2nd-order super-nested arrays
            N1 = 7; N2 = 7;
            LEN_D = 2*((N1+1)*N2-1)+1;
            S = super_nested(N1, N2, 2);
            [n1, n2] = ndgrid(S);
            n1_n2_mat = n1 - n2;
            n1_n2_vec = n1_n2_mat(:);
            D = unique(n1_n2_vec);
            U = D;
            
            title_str = '2nd-order super-nested arrays';
            filename = '2nd_Super_Nested';
            
        case 6
            % 3rd-order super-nested arrays
            N1 = 7; N2 = 7;
            LEN_D = 2*((N1+1)*N2-1)+1;
            S = super_nested(N1, N2, 3);
            [n1, n2] = ndgrid(S);
            n1_n2_mat = n1 - n2;
            n1_n2_vec = n1_n2_mat(:);
            D = unique(n1_n2_vec);
            U = D;
            
            title_str = '3rd-order super-nested arrays';
            filename = '3rd_Super_Nested';
        otherwise
            error('Invalid type');
    end
    
    
    LEN_S = length(S);
    
    % Generate mutual coupling matrix C
    [n1, n2] = ndgrid(S);
    n1_n2_mat = n1 - n2;
    n1_n2_vec = n1_n2_mat(:);
    
    
    c1 = 0.3*exp(1i*pi/3);
    B = 100;
    
%     Coupling matrix
    d_pq = abs(n1_n2_mat);    
    C = c1./d_pq .* exp(-1i*pi/8*(d_pq-1));
    C( d_pq == 0 ) = 1;
    C( d_pq > B ) = 0;
        
    % Array manifold matrix
    V = exp(2i * pi * S * theta_bar');
    
    % Source Process
    Source = diag(amp) * (randn(DD, SNAPSHOTS) + 1i * randn(DD, SNAPSHOTS)) / sqrt(2);
    
    % Noise Process
    noise_std = 10^(-SNRdB/20)*ones(LEN_S, 1) / sqrt(DD);
    Noise = diag(noise_std) * (randn(LEN_S, SNAPSHOTS) + 1i * randn(LEN_S, SNAPSHOTS)) / sqrt(2);
    
    X = C * V * Source + Noise;
    
    
    % Covariance computation
    R_x = X * X' / SNAPSHOTS;
    
    if (type == 1)
        x_U = [R_x(1, end:-1:1).'; R_x(2:end, 1)];
        U = (-LEN_S+1:LEN_S-1).';
    else
        x_SxS = R_x(:);
        x_D = zeros(LEN_D, 1);
        for kk = 1 : LEN_D
            x_D(kk) = mean( x_SxS(n1_n2_vec == D(kk) ) );
        end
        x_U = x_D(D >= min(U) & D <= max(U));
    end
    
    % Plot MUSIC spectrum
    [P_MUSIC, theta_bar_grid] = spectral_MUSIC(x_U, DD);
    theta_bar_est = root_MUSIC( U, x_U, DD );
    
    figure;
    semilogy(   theta_bar_grid, P_MUSIC)
    grid on;
    xlabels = cell(1, DD+2);
    xlabels{1} = '-0.5'; xlabels{DD+2} = '0.5'; 
    xlabels{2} = num2str(-theta_bar_max);
    xlabels{DD+1} = num2str(theta_bar_max);
    set(gca,	'FontSize', SIZE, ...
        'XTick',  [-0.5; theta_bar; 0.5], ...
        'XTickLabel', xlabels, ...
        'YTick', 10.^(-5:0));
    xlabel('$\bar{\theta}$', 'interpreter', 'latex')
    pbaspect(aratio)
    title(title_str)
    
    print('-depsc', [filename, '_simulation__DD_', num2str(DD), '__SNRdB_', num2str(SNRdB), '__Snapshots_', num2str(SNAPSHOTS)]);
    savefig([filename, '_simulation__DD_', num2str(DD), '__SNRdB_', num2str(SNRdB), '__Snapshots_', num2str(SNAPSHOTS)])
    
    f = fopen([filename, '_RMSE__DD_', num2str(DD), '__SNRdB_', num2str(SNRdB), '__Snapshots_', num2str(SNAPSHOTS), '.txt'], 'w');
    fprintf(f, num2str(  norm(theta_bar_est - theta_bar)/sqrt(DD) ));
    fclose(f);
end


